using System;
using System.IO.Ports;
using System.Text;
using System.Threading;

namespace FEZMiniWiFi
{
    static class Wifi
    {
        static SerialPort DebugPort = new SerialPort(Serial.COM1, 115200, Parity.None, 8, StopBits.Two);
        static SerialPort WifiPort = new SerialPort(Serial.COM2, 115200, Parity.None, 8, StopBits.Two);
        static Boolean SocketOpened = false;

        public static void Initialize()
        {
            DebugPort.Open();
            WifiPort.Open();

            Debug("<Initialize>\n");

            //Thread.Sleep(20 * 1000);

            Byte[] rxbuf;
            do
            {
                rxbuf = ReadLine();
                DebugPort.Write(rxbuf, 0, rxbuf.Length);
            }
            while (!(ArrayEquals(rxbuf, UTF8Encoding.UTF8.GetBytes(" Loading Done"), 13)));

            Command("at+rsi_reset");
            Command("at+rsi_band=0");
            Command("at+rsi_init");
            Command("at+rsi_network=IBSS,1,1");
            Command("at+rsi_authmode=0");
            Command("at+rsi_join=PC-WiFi-01,0,1");
            Command("at+rsi_ipconf=0,169.254.1.1,255.255.0.0,0.0.0.0");
        }

        public static void Open()
        {
            Debug("<SocketOpen>\n");
            Command("at+rsi_ltcp=8001");
            SocketOpened = true;
        }

        public static Byte[] Receive()
        {
            if (!SocketOpened) throw new ObjectDisposedException();

            Byte[] rxbuf = new Byte[0];
            while (true)
            {
                ArrayAdd(ref rxbuf, ReadByte());

                if (ArrayEquals(rxbuf, UTF8Encoding.UTF8.GetBytes("AT+RSI_READ"), 11))
                {
                    Debug("<SocketReceive>\n");

                    ReadByte(); /* hn */
                    int sz = ReadByte() | (ReadByte() << 8);

                    Byte[] data = new Byte[0];
                    while (0 < sz)
                    {
                        ArrayAdd(ref data, ReadByte());
                        sz--;
                    }

                    ReadLine();

                    return (data);
                }
                else if (ArrayEquals(rxbuf, UTF8Encoding.UTF8.GetBytes("AT+RSI_CLOSE"), 12))
                {
                    Debug("<SocketClose>\n");
                    ReadLine();
                    SocketOpened = false;
                    throw new ObjectDisposedException();
                }
                else
                {
                    for (int i = 0; i < rxbuf.Length; i++)
                    {
                        Debug(rxbuf[i].ToString() + ", ");
                    }
                    Debug("\n");
                }
            }
        }

        private static void Command(String command)
        {
            WriteLine(UTF8Encoding.UTF8.GetBytes(command));
            Debug(command + "\n");

            Byte[] rxbuf = ReadLine();
            for (int i = 0; i < rxbuf.Length; i++)
            {
                Debug(rxbuf[i].ToString() + ", ");
            }
            Debug("\n");
        }

        private static void WriteByte(Byte value)
        {
            Byte[] txbuf = new Byte[] { value };
            WifiPort.Write(txbuf, 0, 1);
        }

        private static void WriteLine(Byte[] array)
        {
            WifiPort.Write(array, 0, array.Length);
            WriteLine();
        }

        private static void WriteLine()
        {
            Byte[] txbuf = new Byte[] { 0x0D, 0x0A };
            WifiPort.Write(txbuf, 0, txbuf.Length);
        }

        private static Byte ReadByte()
        {
            Byte[] rxbuf = new Byte[1];

            WifiPort.Read(rxbuf, 0, 1);

            return (rxbuf[0]);
        }

        private static Byte[] ReadLine()
        {
            Byte[] rxbuf = new Byte[0];

            do
            {
                ArrayAdd(ref rxbuf, ReadByte());
            }
            while (!((rxbuf[rxbuf.Length - 1] == 0x0A) && (1 < rxbuf.Length) && (rxbuf[rxbuf.Length - 2] == 0x0D)));

            return (rxbuf);
        }

        private static void ArrayAdd(ref Byte[] array, Byte value)
        {
            Byte[] temporary = new Byte[array.Length + 1];

            array.CopyTo(temporary, 0);
            temporary[array.Length] = value;

            array = temporary;
        }

        private static Boolean ArrayEquals(Byte[] arrayA, Byte[] arrayB, int n)
        {
            if ((arrayA.Length < n) || (arrayB.Length < n)) return (false);

            for (int i = 0; i < n; i++)
            {
                if (arrayA[i] != arrayB[i])
                {
                    return (false);
                }
            }

            return (true);
        }

        private static void Debug(String s)
        {
            //Microsoft.SPOT.Debug.Print(s);
            Byte[] txbuf = UTF8Encoding.UTF8.GetBytes(s);
            DebugPort.Write(txbuf, 0, txbuf.Length);
        }
    }
}
